//
//  CertificateManager.swift
//  caclient
//
//  Created by BCC CA on 17/4/23.
//

import Foundation

@objc class CertificateManager : NSObject {
    private static var _instance:CertificateManager? = nil;

    @objc public static func getInstance() -> CertificateManager {
        if _instance == nil {
            _instance = CertificateManager();
        }
        
        return _instance!;
    }

    override init() {
        print("initiated CertificateManager")
    }
    
    @objc public func generateCSR(_ enrollmentId: String, commonName: String, organization: String, organizationUnit: String, country: String, state: String, area: String, postalCode: String, serialNumber: String) -> String? {
        
        //let emailAddress = emailAddress as String; // Is not being used
        let enrollmentId = enrollmentId as String
        let commonName = commonName as String
        let organization = organization as String
        let organizationUnit = organizationUnit as String
        let country = country as String
        let state = state as String
        let area = area as String
        let postalCode = postalCode as String
        let serialNumber = serialNumber as String
        
        let keypair = KeyPairManager.getInstance();
        print("inside function")
        let publicKey = keypair.retrievePublicKey(enrollmentId: enrollmentId);
        if publicKey == nil {
            print("public key is nil")
            return nil;
        }
        
        let privateKeySignedResponse = keypair.retrievePrivateKey(enrollmentId: enrollmentId)
        if privateKeySignedResponse.getStatus() == errSecItemNotFound {
            return nil;
        }
        let privatekeyData = SecKeyCopyExternalRepresentation(privateKeySignedResponse.getSecKeyReference(), nil) as! Data
        
        //print("private key is" + privatekeyData.base64EncodedString())
        
        let csr = CertificateSigningRequest(commonName: commonName, organizationName: organization, organizationUnitName: organizationUnit, countryName: country, stateOrProvinceName: state, localityName: area, serialNumber: serialNumber, keyAlgorithm: Constants.KEY_ALGORITHM)
        // Build the CSR
        let publickeyBits = keypair.getPublicKeyBits(publicKey: publicKey!, enrollmentId: enrollmentId).0;
        
        let csrBuild = csr.buildAndEncodeDataAsString(publickeyBits!, privateKey: privateKeySignedResponse.getSecKeyReference(), publicKey: publicKey)
        if let csrRegular = csrBuild {
            print("CSR string no header and footer")
            print(csrRegular)
            print(csrBuild!.count, 0, "CSR contains no data")
        } else {
            print(csrBuild, "CSR with header not generated")
        }
        //print(csrBuild)
        return csrBuild
    }
}
