//
//  KeyPairManager.swift
//  caclient
//
//  Created by BCC CA on 17/4/23.
//

import Foundation

@objc class KeyPairManager: NSObject {
    
    private static var _instance:KeyPairManager? = nil;
    
    @objc public static func getInstance() -> KeyPairManager {
        if _instance == nil {
            _instance = KeyPairManager();
        }
        
        return _instance!;
    }
    
    override init() {
        print("KeyPairManager initiated")
    }
    
    public func getKeyPair(_ algorithm: KeyAlgorithm, keySize: Int, tagPrivate: String, tagPublic: String)->(SecKey?,SecKey?) {
        let publicKeyParameters: [String : Any] = [
            String(kSecAttrIsPermanent): kCFBooleanTrue,
            String(kSecAttrApplicationTag): tagPublic.data(using: .utf8),
            String(kSecAttrAccessible): kSecAttrAccessibleWhenUnlockedThisDeviceOnly,
        ]
        
        let privateKeyParameters: [String: Any] = [
            String(kSecAttrIsPermanent) : kCFBooleanTrue,
            String(kSecAttrApplicationTag) : tagPrivate.data(using: .utf8),
            String(kSecAttrAccessible): kSecAttrAccessibleWhenUnlockedThisDeviceOnly,
        ]
        
        let parameters : [String: AnyObject] = [
            String(kSecAttrKeyType): algorithm.secKeyAttrType,
            String(kSecAttrKeySizeInBits): keySize as AnyObject,
            //String(kSecReturnRef): kCFBooleanTrue,
            kSecPublicKeyAttrs as String: publicKeyParameters as AnyObject,
            kSecPrivateKeyAttrs as String: privateKeyParameters as AnyObject,
        ]
        
        var error: Unmanaged<CFError>?
        let privateKey = SecKeyCreateRandomKey(parameters as CFDictionary, &error)
        if privateKey == nil {
            print("Error creating keys occured: \(error!.takeRetainedValue() as Error), keys were not created")
            //return(nil,nil)
        }
        
        let query: [String : AnyObject] = [
            String(kSecClass): kSecClassKey,
            String(kSecAttrKeyType): algorithm.secKeyAttrType,
            String(kSecAttrApplicationTag): tagPublic as AnyObject,
            String(kSecReturnRef): kCFBooleanTrue
        ]
        guard let publicKey = SecKeyCopyPublicKey(privateKey!) else{
            print("cannot get public key")
            return (privateKey,nil)
            // return (privateKey,nil)
        }
        let publickeyData = SecKeyCopyExternalRepresentation(publicKey, nil)! as Data
        
        print(publickeyData.base64EncodedString())
        return (privateKey,publicKey)

    }
    
//    func generateKeyPairInKeyChain(_ algorithm: KeyAlgorithm, keySize: Int, tagPrivate: String, tagPublic: String, enrollmentId: String) -> String {
//
//        let privateAttributes = [String(kSecAttrIsPermanent) : kCFBooleanTrue,
//                                 String(kSecAttrApplicationTag) : tagPrivate.data(using: .utf8),
//                                 String(kSecAttrAccessible): kSecAttrAccessibleWhenUnlockedThisDeviceOnly,] as [String : Any]
//        let publicAttributes = [String(kSecAttrIsPermanent): kCFBooleanTrue,
//                                String(kSecAttrApplicationTag): tagPublic.data(using: .utf8),
//                                String(kSecAttrAccessible): kSecAttrAccessibleWhenUnlockedThisDeviceOnly,] as [String : Any]
//
//        let pairAttributes = [String(kSecAttrKeyType): algorithm.secKeyAttrType,
//                              String(kSecAttrKeySizeInBits): keySize as AnyObject,
//                              //String(kSecReturnRef): kCFBooleanTrue,
//                              kSecPublicKeyAttrs as String: publicAttributes as AnyObject,
//                              kSecPrivateKeyAttrs as String: privateAttributes as AnyObject,]
//
//        //Ensure that keychain has no key with keyId identifier
//        deleteKeyPairFromKeyChain(enrollmentId: enrollmentId)
//
//        var publicRef: SecKey?
//        var privateRef: SecKey?
//
//        //Generate the keys and recover the references in publicRef and privateRf
//        switch SecKeyGeneratePair(pairAttributes as CFDictionary, &publicRef, &privateRef) {
//            case noErr:
//                // Get the public key as a String
//                let publicKeyStr = loadKeyStringFromKeyChain(enrollmentId: enrollmentId)
//                return publicKeyStr
//            default:
//                return ""
//        }
//    }
//    
//    func deleteKeyPairFromKeyChain(enrollmentId: String) {
//        deleteRSAKeyFromKeychain(enrollmentId: enrollmentId)
//        deleteRSAKeyFromKeychain(enrollmentId: enrollmentId)
//    }
//
//    // Delete existing RSA key from keychain
//    private func deleteRSAKeyFromKeychain(enrollmentId: String) {
//        let queryFilter: [String: Any] = [
//            String(kSecClass): kSecClassKey,
//            String(kSecAttrKeyType): Constants.KEY_ALGORITHM.secKeyAttrType,
//            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
//            String(kSecReturnRef): true
//        ]
//        let status: OSStatus = SecItemDelete(queryFilter as CFDictionary)
//        NSLog("private or public deletion result is: " + status.description)
//    }
//    
//    func loadKeyStringFromKeyChain(enrollmentId: String) -> String {
//        let query: Dictionary<String, Any> = [
//            String(kSecClass): kSecClassKey,
//            String(kSecAttrKeyType): Constants.KEY_ALGORITHM.secKeyAttrType,
//            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
//            String(kSecAttrKeySizeInBits): Constants.KEY_SIZE,
//            kSecReturnData as String : kCFBooleanTrue]
//
//        var dataTypeRef: AnyObject? = nil
//        var resultData: NSData? = nil
//
//        let status: OSStatus = withUnsafeMutablePointer(to: &dataTypeRef) { SecItemCopyMatching(query as NSDictionary, UnsafeMutablePointer($0)) }
//        NSLog("SecItemCopyMatching: " + status.description)
//
//        if status == errSecSuccess {
//            NSLog("private or public debug description is: " + dataTypeRef.debugDescription)
//            resultData = dataTypeRef as? NSData
//            let resultStr = resultData?.base64EncodedString(options: [])
//            NSLog("private or public String is: " + resultStr!)
//            return resultStr!
//        } else {
//            NSLog("no key found!!!!")
//            return ""
//        }
//    }
//    
//    private func loadKeyStringFromKeyChainAsNSData(enrollmentId: String) -> NSData {
//        let query: Dictionary<String, Any> = [
//            String(kSecAttrKeyType): kSecAttrKeyTypeRSA,
//            String(kSecAttrKeySizeInBits): Constants.KEY_SIZE,
//            String(kSecClass): kSecClassKey,
//            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
//            kSecReturnData as String : kCFBooleanTrue ]
//
//        var dataTypeRef: AnyObject? = nil
//        var resultData: NSData? = nil
//
//        let status: OSStatus = withUnsafeMutablePointer(to: &dataTypeRef) { SecItemCopyMatching(query as NSDictionary, UnsafeMutablePointer($0)) }
//        NSLog("SecItemCopyMatching: " + status.description)
//
//        if status == errSecSuccess {
//            NSLog("private or public debug description is: " + dataTypeRef.debugDescription)
//            resultData = dataTypeRef as? NSData
//            return resultData!
//        } else {
//            NSLog("no key found!!!!")
//            return resultData!
//        }
//    }
//    
    public func retrievePublicKey(enrollmentId: String) -> SecKey? {
        
        var error: Unmanaged<CFError>?

        // Get generated public key
        let query: [String: Any] = [
            String(kSecClass): kSecClassKey,
            String(kSecAttrKeyType): Constants.KEY_ALGORITHM.secKeyAttrType,
            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
            String(kSecReturnRef): true
        ]
        
        var publicKeyReturn: CFTypeRef?
        let result = SecItemCopyMatching(query as CFDictionary, &publicKeyReturn)
        if result != errSecSuccess {
            print("Error getting publicKey fron keychain occured: \(result)")
            return nil
        }
        // swiftlint:disable:next force_cast
        let publicKey = publicKeyReturn as! SecKey?
        return publicKey
        
//        var dataPtr:CFTypeRef? = nil
//        // Get generated public key
//        let query: [String: Any] = [
//            String(kSecClass): kSecClassKey,
//            String(kSecAttrKeyType): Constants.KEY_ALGORITHM.secKeyAttrType,
//            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
//            String(kSecReturnRef): true
//        ]
//        let qResult = SecItemCopyMatching(query as CFDictionary, &dataPtr)
//
//        // error handling with `qResult` ...
//
//
//        if qResult != errSecSuccess {
//                    print("Error getting publicKey from keychain occured: \(qResult)")
//                    return nil
//                }
//        let data = dataPtr as! Data
//        let base64PublicKey = data.base64EncodedString()
//        print(base64PublicKey)
        
//        var error: Unmanaged<CFError>?
//
//
//
//        var publicKeyReturn: CFTypeRef?
//        let result = SecItemCopyMatching(query as CFDictionary, &publicKeyReturn)
//        if result != errSecSuccess {
//            print("Error getting publicKey from keychain occured: \(result)")
//            return nil
//        }
//        guard let secKeyData = Data.init(base64Encoded: base64PublicKey) else {
//                print("Error: invalid encodedKey, cannot extract data")
//                return nil
//            }
//        guard let publicKey = SecKeyCreateWithData(secKeyData as CFData, query as CFDictionary, nil) else {
//            print("Error: Problem in SecKeyCreateWithData()")
//            return nil
//        }
//
//        return publicKey
    }
    
    public func retrievePrivateKey(enrollmentId: String) -> SignedResponse {
        let query: [String: Any] = [
          kSecClass as String: kSecClassKey,
          kSecAttrKeyClass as String: kSecAttrKeyClassPrivate,
          kSecAttrApplicationTag as String:  (Constants.CERT_TAG_PRIVATE  + String(enrollmentId)).data(using: .utf8)!,
          kSecReturnRef as String: true,
          // kSecUseOperationPrompt as String:"Hello There"
        ]

        var data: AnyObject?
        let status = SecItemCopyMatching(query as CFDictionary, &data)

        let signedResponse:SignedResponse = SignedResponse()

        signedResponse.setStatus(s: status)
        if(status == errSecSuccess) {
            signedResponse.setSecKeyReference(sec: (data as! SecKey?)!)
        }
        return signedResponse;
   }
    enum KeychainError: Error {
        case failedToRetrievePrivateKey
    }
    
    public func getPublicKeyBits(publicKey: SecKey, enrollmentId: String)->(Data?,Int?) {
        
        let keyBlockSize = SecKeyGetBlockSize(publicKey)
        // Ask keychain to provide the publicKey in bits
        let query: [String: Any] = [
            String(kSecClass): kSecClassKey,
            String(kSecAttrKeyType): Constants.KEY_ALGORITHM.secKeyAttrType,
            String(kSecAttrApplicationTag): (Constants.CERT_TAG_PUBLIC + String(enrollmentId)).data(using: .utf8)!,
            String(kSecReturnData): true
        ]
        
        var tempPublicKeyBits: CFTypeRef?
        var _ = SecItemCopyMatching(query as CFDictionary, &tempPublicKeyBits)
        
        guard let keyBits = tempPublicKeyBits as? Data else {
            return (nil, nil)
        }
        
        return (keyBits, keyBlockSize)
    }
}

